"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Groups the elements of an array based on the given function and
 * returns the count of elements in each group
 * @param array
 * @param fn
 * @example
 * countBy([6.1, 4.2, 6.3], Math.floor); // { 4: 1, 6: 2 }
 * @example
 * countBy(['one', 'two', 'three'], 'length'); // { 3: 2, 5: 1 }
 */
function countBy(array, fn) {
    return array
        .map(typeof fn === "function" ? fn : (value) => value[fn])
        .reduce((prev, value) => {
        prev[value] = (prev[value] || 0) + 1;
        return prev;
    }, {});
}
exports.default = countBy;
//# sourceMappingURL=index.js.map