"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_js_1 = __importDefault(require("../../string/isString/index.js"));
const utils_js_1 = require("../../utils.js");
/**
 * Returns all the distinct values of an array
 * @param array
 * @param [fn]
 * @example
 * distinct([1, 2, 2, 3, 4, 4, 5]); // [1,2,3,4,5]
 * @example
 * distinct([{foo:{bar:[0,1]}}, {foo:{bar:[4,1]}}], "foo.bar[1]"); // [{foo:{bar:[0,1]}}]
 * @example
 * distinct([1, 2, 2.2, 3, 4.9, 4, 5], Math.floor); // [1,2,3,4.9,5]
 */
function distinct(array, fn) {
    if (!fn)
        return [...new Set(array)];
    if ((0, index_js_1.default)(fn)) {
        const keys = (0, utils_js_1.pathToKeys)(fn);
        // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
        fn = value => keys.reduce((prev, cur) => prev[cur], value);
    }
    const length = array.length;
    const found = {};
    const result = [];
    for (let i = 0; i < length; i++) {
        const value = array[i];
        const generatedValue = fn(value, i, array);
        if (found[generatedValue])
            continue;
        found[generatedValue] = true;
        result.push(value);
    }
    return result;
}
exports.default = distinct;
//# sourceMappingURL=index.js.map