"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_js_1 = __importDefault(require("../../string/isString/index.js"));
const utils_js_1 = require("../../utils.js");
/**
 * Groups the elements of an array based on the given function and
 * returns the count of elements in each group
 * @param arr
 * @param fn
 * @example
 * sortBy(['five', 'three', 'one'], 'length'); // ['one', 'five', 'three']
 * @example
 * sortBy(['five', 'three', 'one'], (value) => -value.length); // ['three', 'five', 'one']
 */
function sortBy(arr, fn) {
    let reducer = fn;
    if ((0, index_js_1.default)(fn)) {
        const keys = (0, utils_js_1.pathToKeys)(fn);
        reducer = (value) => keys.reduce((prev, cur) => prev[cur], value);
    }
    return arr.sort((a, b) => reducer(a) - reducer(b));
}
exports.default = sortBy;
//# sourceMappingURL=index.js.map