"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_js_1 = __importDefault(require("../values/index.js"));
/**
 * Returns the target value in a nested JSON object, based on the given key
 * @param obj
 * @param target
 * @example
 * const data = {
 *  level1: {
 *   level2: {
 *    level3: 'some data'
 *   }
 *  }
 * };
 * dig(data, 'level3'); // 'some data'
 * dig(data, 'level4'); // undefined
 */
function dig(obj, target) {
    if (target in obj)
        return obj[target];
    return (0, index_js_1.default)(obj).reduce((acc, val) => {
        if (acc != null)
            return acc;
        if (typeof val === "object")
            return dig(val, target);
        // eslint-disable-next-line no-undefined
    }, undefined);
}
exports.default = dig;
//# sourceMappingURL=index.js.map