"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_js_1 = require("../../utils.js");
const index_js_1 = __importDefault(require("../keys/index.js"));
/**
 * Creates a new object from the combination of two or more objects
 * @param obj
 * @param objects
 * @example
 * const object = {
 *   a: [{ x: 2 }, { y: 4 }],
 *   b: 1
 * };
 * const other = {
 *   a: { z: 3 },
 *   b: [2, 3],
 *   c: "foo"
 * };
 * merge(object, other); // { a: [ { x: 2 }, { y: 4 }, { z: 3 } ], b: [ 1, 2, 3 ], c: "foo" }
 */
function merge(obj, ...objects) {
    return [obj, ...objects].reduce((prev, obj2) => (0, index_js_1.default)(obj2).reduce((a, k) => {
        prev[k] = utils_js_1.hasOwnProperty.call(prev, k)
            ? [].concat(prev[k]).concat(obj2[k])
            : obj2[k];
        return prev;
    }, {}), {});
}
exports.default = merge;
//# sourceMappingURL=index.js.map