"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_js_1 = __importDefault(require("../keys/index.js"));
/**
 * Omits the key-value pairs corresponding to the given keys from an object; or
 * Creates an object composed of the properties the given function returns falsey for.
 * The function is invoked with two arguments: (value, key)
 * @param obj
 * @param arr
 * @example
 * omit({ a: 1, b: '2', c: 3 }, ['b']); // { a: 1, c: 3 }
 * @example
 * omit({ a: 1, b: '2', c: 3 }, (x) => typeof x === 'number'); // { b: '2' }
 */
function omit(obj, arr) {
    let fn = arr;
    if (Array.isArray(arr))
        fn = (v, k) => arr.includes(k);
    return (0, index_js_1.default)(obj)
        .filter(k => !fn(obj[k], k, obj))
        .reduce((acc, key) => {
        acc[key] = obj[key];
        return acc;
    }, {});
}
exports.default = omit;
//# sourceMappingURL=index.js.map