export declare const STRAIGHT_ANGLE_DEGREE = 180;
export declare const PI: number;
export declare const hasOwnProperty: (v: PropertyKey) => boolean;
/**
 * Convert key path string to key array
 * @private
 * @param path
 * @example
 * pathToKeys("selector.to[0][11].value"); // ["selector", "to", 0, 11, "value"]
 */
export declare const pathToKeys: (path: string) => Array<number | string>;
/**
 *
 * @private
 * @param obj
 * @param keys
 * @param method
 * @example
 * addPrototype(Object, "$size", function() {return this;});
 */
export declare const addPrototype: (obj: unknown, keys: string[] | string, method: (...args: any[]) => any) => void;
/**
 *
 * @private
 * @param arr
 * @param [path]
 * @param fn
 * @example
 * filter([1, 2, 3], undefined, (value) => value > 1);
 */
export declare const filter: <T>(arr: T[], path: string | undefined, fn: (value: T, index: number, array: T[]) => any) => T[];
/**
 *
 * @private
 * @param value
 * @example
 * deepClone([1, 2, 3]);
 */
export declare const deepClone: <T>(value: T) => T;
export declare type OperatorT = "<" | "<=" | "<>" | "=" | ">" | ">=";
export declare const enum OPERATOR {
    LT = "<",
    LTE = "<=",
    EQ = "=",
    NE = "<>",
    GTE = ">=",
    GT = ">"
}
export declare type RecordT = Record<string, any>;
export declare type PathsToStringPropsT<Value> = Value extends Record<string, unknown> ? {
    [Key in keyof Value]: [Key, ...PathsToStringPropsT<Value[Key]>] | [Key];
}[keyof Value] : [];
export declare type JoinT<Value extends string[]> = Value extends [] ? never : Value extends [infer F] ? F : Value extends [infer F, ...infer R] ? F extends string ? string extends F ? string : `${F}.${JoinT<Extract<R, string[]>>}` : never : string;
export declare type PathT<Value> = Value extends RecordT ? JoinT<Extract<PathsToStringPropsT<Value>, string[]>> : never;
