"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OPERATOR = exports.deepClone = exports.filter = exports.addPrototype = exports.pathToKeys = exports.hasOwnProperty = exports.PI = exports.STRAIGHT_ANGLE_DEGREE = void 0;
exports.STRAIGHT_ANGLE_DEGREE = 180.0;
exports.PI = Math.PI;
exports.hasOwnProperty = Object.prototype.hasOwnProperty;
/**
 * Convert key path string to key array
 * @private
 * @param path
 * @example
 * pathToKeys("selector.to[0][11].value"); // ["selector", "to", 0, 11, "value"]
 */
const pathToKeys = (path) => path
    .replace(/(?<key>[^.])\[/g, (m, match) => `${match}.[`)
    .split(".")
    .map((key) => {
    const match = key.match(/^\[(?<digits>\d+)]$/);
    if (match?.groups)
        return +match.groups.digits;
    return key;
});
exports.pathToKeys = pathToKeys;
/**
 *
 * @private
 * @param obj
 * @param keys
 * @param method
 * @example
 * addPrototype(Object, "$size", function() {return this;});
 */
const addPrototype = (obj, keys, method) => {
    if (Array.isArray(keys)) {
        keys.forEach(key => (0, exports.addPrototype)(obj, key, method));
        return;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const prototype = obj.prototype;
    if (exports.hasOwnProperty.call(prototype, keys))
        return;
    Object.defineProperty(prototype, keys, {
        value(...args) {
            return method.apply(0, [this].concat(args));
        },
        writable: true,
    });
};
exports.addPrototype = addPrototype;
/**
 *
 * @private
 * @param arr
 * @param [path]
 * @param fn
 * @example
 * filter([1, 2, 3], undefined, (value) => value > 1);
 */
const filter = (arr, path, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
fn) => {
    if (path) {
        const keys = (0, exports.pathToKeys)(path);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const reducer = (item) => keys.reduce((prev, curr) => prev[curr], item);
        return arr.filter((item, index, items) => fn(reducer(item), index, items));
    }
    return arr.filter(fn);
};
exports.filter = filter;
/**
 *
 * @private
 * @param value
 * @example
 * deepClone([1, 2, 3]);
 */
const deepClone = (value) => {
    if (typeof value !== "object")
        return value;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    if (value instanceof Date)
        return new Date(value.getTime());
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const clone = { ...value };
    Object.keys(clone).forEach(key => (clone[key]
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        = typeof value[key] === "object"
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            ? (0, exports.deepClone)(value[key])
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            : value[key]));
    if (Array.isArray(value)) {
        clone.length = value.length;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return Array.from(clone);
    }
    return clone;
};
exports.deepClone = deepClone;
var OPERATOR;
(function (OPERATOR) {
    OPERATOR["LT"] = "<";
    OPERATOR["LTE"] = "<=";
    OPERATOR["EQ"] = "=";
    OPERATOR["NE"] = "<>";
    OPERATOR["GTE"] = ">=";
    OPERATOR["GT"] = ">";
})(OPERATOR = exports.OPERATOR || (exports.OPERATOR = {}));
//# sourceMappingURL=utils.js.map