import { PathT } from "../../utils.js";
declare const enum ORDER {
    ASC = "asc",
    DESC = "desc"
}
/**
 * Sorts the array
 * @param arr
 * @param [order="asc"]
 * @example
 * orderBy([2, 1, 2, 5]); // [1,2,2,5]
 * @example
 * orderBy([2, 1, 2, 5], "desc"); // [5,2,2,1]
 */
declare function orderBy<Value>(arr: Value[], order?: OrderT): Value[];
/**
 * Sorts the array
 * @param array
 * @param field
 * @param [order="asc"]
 * @example
 * orderBy([{count:1}, {count:20}, {count:15}], "count", "asc"); // [{count:1},{count:15},{count:20}]
 */
declare function orderBy<Value extends Record<string, unknown>, Path extends PathT<Value> = never>(array: Value[], field: Path, order?: OrderT): Value[];
export default orderBy;
export { ORDER };
export declare type OrderT = "asc" | "desc";
