import isString from "../../string/isString/index.js";
import { pathToKeys } from "../../utils.js";
/**
 * Groups the elements of an array based on the given function and
 * returns the count of elements in each group
 * @param arr
 * @param fn
 * @example
 * sortBy(['five', 'three', 'one'], 'length'); // ['one', 'five', 'three']
 * @example
 * sortBy(['five', 'three', 'one'], (value) => -value.length); // ['three', 'five', 'one']
 */
export default function sortBy(arr, fn) {
    let reducer = fn;
    if (isString(fn)) {
        const keys = pathToKeys(fn);
        reducer = (value) => keys.reduce((prev, cur) => prev[cur], value);
    }
    return arr.sort((a, b) => reducer(a) - reducer(b));
}
//# sourceMappingURL=index.js.map