import forEach from "../forEach/index.js";
import isPlainObject from "../isPlainObject/index.js";
/**
 * Flattens the object into a single-depth object
 * @param obj
 * @example
 * flatten({ foo: 1, bar: { foo: 2 } }); // { foo: 1, "bar.foo": 2 }
 */
export default function flatten(obj) {
    const result = {};
    forEach(obj, (value, key) => {
        if (Array.isArray(value)) {
            const temp = {};
            value.forEach((subValue, index) => {
                temp[`[${index}]`] = subValue;
            });
            value = temp;
        }
        if (isPlainObject(value)) {
            const flat = flatten(value);
            forEach(flat, (subValue, subKey) => {
                const separator = subKey.startsWith("[") ? "" : ".";
                result[`${key}${separator}${subKey}`] = subValue;
            });
            return;
        }
        result[key] = value;
    });
    return result;
}
//# sourceMappingURL=index.js.map