import { hasOwnProperty } from "../../utils.js";
import keys from "../keys/index.js";
/**
 * Creates a new object from the combination of two or more objects
 * @param obj
 * @param objects
 * @example
 * const object = {
 *   a: [{ x: 2 }, { y: 4 }],
 *   b: 1
 * };
 * const other = {
 *   a: { z: 3 },
 *   b: [2, 3],
 *   c: "foo"
 * };
 * merge(object, other); // { a: [ { x: 2 }, { y: 4 }, { z: 3 } ], b: [ 1, 2, 3 ], c: "foo" }
 */
export default function merge(obj, ...objects) {
    return [obj, ...objects].reduce((prev, obj2) => keys(obj2).reduce((a, k) => {
        prev[k] = hasOwnProperty.call(prev, k)
            ? [].concat(prev[k]).concat(obj2[k])
            : obj2[k];
        return prev;
    }, {}), {});
}
//# sourceMappingURL=index.js.map