export const STRAIGHT_ANGLE_DEGREE = 180.0;
export const PI = Math.PI;
export const hasOwnProperty = Object.prototype.hasOwnProperty;
/**
 * Convert key path string to key array
 * @private
 * @param path
 * @example
 * pathToKeys("selector.to[0][11].value"); // ["selector", "to", 0, 11, "value"]
 */
export const pathToKeys = (path) => path
    .replace(/(?<key>[^.])\[/g, (m, match) => `${match}.[`)
    .split(".")
    .map((key) => {
    const match = key.match(/^\[(?<digits>\d+)]$/);
    if (match?.groups)
        return +match.groups.digits;
    return key;
});
/**
 *
 * @private
 * @param obj
 * @param keys
 * @param method
 * @example
 * addPrototype(Object, "$size", function() {return this;});
 */
export const addPrototype = (obj, keys, method) => {
    if (Array.isArray(keys)) {
        keys.forEach(key => addPrototype(obj, key, method));
        return;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const prototype = obj.prototype;
    if (hasOwnProperty.call(prototype, keys))
        return;
    Object.defineProperty(prototype, keys, {
        value(...args) {
            return method.apply(0, [this].concat(args));
        },
        writable: true,
    });
};
/**
 *
 * @private
 * @param arr
 * @param [path]
 * @param fn
 * @example
 * filter([1, 2, 3], undefined, (value) => value > 1);
 */
export const filter = (arr, path, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
fn) => {
    if (path) {
        const keys = pathToKeys(path);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const reducer = (item) => keys.reduce((prev, curr) => prev[curr], item);
        return arr.filter((item, index, items) => fn(reducer(item), index, items));
    }
    return arr.filter(fn);
};
/**
 *
 * @private
 * @param value
 * @example
 * deepClone([1, 2, 3]);
 */
export const deepClone = (value) => {
    if (typeof value !== "object")
        return value;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    if (value instanceof Date)
        return new Date(value.getTime());
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const clone = { ...value };
    Object.keys(clone).forEach(key => (clone[key]
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        = typeof value[key] === "object"
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            ? deepClone(value[key])
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            : value[key]));
    if (Array.isArray(value)) {
        clone.length = value.length;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return Array.from(clone);
    }
    return clone;
};
export var OPERATOR;
(function (OPERATOR) {
    OPERATOR["LT"] = "<";
    OPERATOR["LTE"] = "<=";
    OPERATOR["EQ"] = "=";
    OPERATOR["NE"] = "<>";
    OPERATOR["GTE"] = ">=";
    OPERATOR["GT"] = ">";
})(OPERATOR || (OPERATOR = {}));
//# sourceMappingURL=utils.js.map